@@ -37,6 +37,10 @@ class Experiment(Base, ArtifactMixin, DataframeMixin, TagMixin):
         self._features = []
         self._parameters = []
 
+    def _get_identifiers(self):
+        """Get the experiment's project's name and the experiment's ID."""
+        return self.project.name, self.id
+
     def log_metric(self, name, value, directionality="score", description=None):
         """Create a metric under the experiment.
 
